#!/usr/bin/env Rscript --vanilla

# Plot coverage histograms generated by e.g. 'mccortex31 clean --length-before out.csv ...'
# input csv should have the columns: 'bp' and 'Count'
#
args <- commandArgs(trailingOnly=TRUE)
if(length(args) != 2) {
  stop("Usage: Rscript --vanilla plot-hist-hist.R <lengths.csv> <lengths.pdf>\n")
}

input_csv=args[1]
output_pdf=args[2]

library('ggplot2')

d=read.csv(file=input_csv,sep=',',as.is=T)

p <- ggplot(data=d, aes(bp, Count)) +
       geom_bar(stat="identity", color="seagreen") +
       scale_y_log10() +
       xlab("Untig length (bases)") +
       ylab("Number of unitigs") +
       ggtitle("Unitig length distribution") +
       xlim(0,75)

ggsave(filename=output_pdf, plot=p, width=6, height=6)
